@@ -100,7 +100,7 @@ def _get_stack(nflxprofile_nodes, node_id, has_node_stack=False, pid_comm=None,
             stack_frame.libtype = nflxprofile_nodes[node_id].libtype
             stack.append(stack_frame)
         if inverted:
-            return reversed(stack)
+            return stack[::-1]
         return stack
 
     # has node stack calculated, returning that
@@ -114,7 +114,7 @@ def _get_stack(nflxprofile_nodes, node_id, has_node_stack=False, pid_comm=None,
         stack_frame.libtype = nflxprofile_nodes[node_id].libtype
         stack = [stack_frame] + list(nflxprofile_nodes[node_id].stack)
         if inverted:
-            return reversed(stack)
+            return stack[::-1]
         return stack
 
     # need to use parent id
@@ -255,19 +255,23 @@ class JavaStackProcessor(StackProcessor):
         processed_frame = nflxprofile_pb2.StackFrame()
         processed_frame.CopyFrom(frame)
         name = frame.function_name
+        name_parts = name.split('::')
 
-        name = name.split('::')[0]
-        name = name.split('$$')[0]
+        class_name = name_parts[0]
+        class_name = class_name.split('$$')[0]
 
-        if frame.libtype and frame.libtype == 'jit' and name.startswith("L"):
-            name = name[1:]
+        if frame.libtype and frame.libtype in ['jit', 'inlined'] and class_name.startswith("L"):
+            class_name = class_name[1:]
 
-        if name.endswith(';'):
-            name = name[:-1]
+        if class_name.endswith(';'):
+            class_name = class_name[:-1]
 
-        name = name.replace('/', '.')
+        class_name = class_name.replace('/', '.')
 
-        processed_frame.function_name = name
+        if len(name_parts) > 1:
+            processed_frame.function_name = class_name + "::" + name_parts[1]
+        else:
+            processed_frame.function_name = class_name
 
         return processed_frame, FrameExtras()
 
@@ -426,10 +430,36 @@ class NodeJsStackProcessor(StackProcessor):
             if name and name[0] == '*':
                 name = name[1:]
 
+            # The nodejs-JIT captured name may have multiple splits, in the case of classes
+            # such as
+            #
+            # class User {
+            #    get email() { ... }
+            # }
+            #
+            # this shows up as LazyCompile:get email /path/to/source.js:xx
+
+            # "Function names can contain letters, digits, underscores, and dollar signs"
+
+            # So, here, we detect the filename field based on the presence
+            # of any function-name characters
+
             if " " in name:
-                frame.file.file_name = name[name.index(" ") + 1:]
-                if frame.file.file_name:
-                    name = name[:name.index(" ")]
+                in_file_mode = False
+                name_parts = name.split(" ")
+                name = ""
+                frame.file.file_name = ""
+                for name_part in name_parts:
+                    if "/" in name_part or ":" in name_part:
+                        in_file_mode = True
+
+                    if in_file_mode:
+                        frame.file.file_name += name_part + " "
+                    else:
+                        name += name_part + " "
+
+                frame.file.file_name = frame.file.file_name[:-1]
+                name = name[:-1]
             processed_frame.function_name = name or "(anonymous)"
 
         return processed_frame, frame_extras
@@ -560,7 +590,7 @@ def get_flame_graph(profile, pid_comm, **args):
     samples = profile.samples
     time_deltas = profile.time_deltas
     start_time = profile.start_time
-    current_time = start_time + time_deltas[0]
+    current_time = start_time
 
     has_samples_cpu = \
         'has_samples_cpu' in profile.params and profile.params['has_samples_cpu'] == 'true'
@@ -599,10 +629,7 @@ def get_flame_graph(profile, pid_comm, **args):
 
     aggregated_samples = {}
     for index, sample in enumerate(samples):
-        if index == (len(samples) - 1):  # last sample
-            break
-        delta = time_deltas[index + 1]
-        current_time += delta
+        current_time += time_deltas[index]
 
         should_skip = False
         for sample_filter in sample_filters:
@@ -633,7 +660,7 @@ def get_flame_graph(profile, pid_comm, **args):
         sample_value = aggregated_samples[sample_id]
 
         if stacks:
-            stack = stacks[sample_id] if not inverted else reversed(stacks[sample_id])
+            stack = stacks[sample_id] if not inverted else stacks[sample_id][::-1]
         else:
             stack = _get_stack(nodes, sample_id, has_node_stack, pid_comm, **args)
 
